/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.basher.internal.impl;

import net.sourceforge.basher.*;
import net.sourceforge.basher.events.PhaseTransitionEvent;
import net.sourceforge.basher.internal.Randomizer;
import net.sourceforge.basher.internal.TaskInvoker;
import net.sourceforge.basher.tasks.ATestTask;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;

/**
 * @author Johan Lindquist
 * @version 1.0
 */
public class TestTaskRunnerImpl extends BasherTestCase
{
    private Log _log;
    private Randomizer _randomizer;
    private TaskManager _taskManager;
    private TaskInvoker _taskInvoker;
    public BasherContext _basherContext;
    public TaskRunnerImpl _taskRunnerImpl;


    private void initializeControls()
    {
        _log = createMock( Log.class );
        _taskInvoker = createMock( TaskInvoker.class );
        _randomizer = createMock(Randomizer.class);
        _taskManager = createMock(TaskManager.class);
    }


    public void testInvoke()
    {
        setupBasherContext(_taskRunnerImpl,500,0);

        final ATestTask aTestTask = new ATestTask();

        final TaskExecutionContext executionContext = new TaskExecutionContext( aTestTask );
        expect(_taskManager.getNextTaskExecutionContext()).andReturn( executionContext );

        _taskInvoker.invokeTask( executionContext );


        expect(_randomizer.getRandomInt(500)).andReturn(1000);


        replayAll();

        Thread t = new Thread()
        {
            public void run()
            {
                try
                {
                    Thread.sleep(500);
                }
                catch (InterruptedException e)
                {
                    e.printStackTrace();
                }
                _taskRunnerImpl.stopInvoking();
            }
        };

        t.start();
        _taskRunnerImpl.run();

        verifyAll();

    }

    private void setupBasherContext(final TaskRunnerImpl taskRunner, int maxDelay, int minTime)
    {
        final BasherContext basherContext = new BasherContext();
        basherContext.setTaskMaxDelay(maxDelay);
        basherContext.setTaskMinDelay(minTime);
        basherContext.setInitialNumberThreads(1);
        taskRunner.basherEvent(new PhaseTransitionEvent(basherContext, null,Phase.START, 0 ));
        taskRunner.basherEvent(new PhaseTransitionEvent(basherContext, Phase.START,Phase.SETUP, 0 ));
        taskRunner.basherEvent(new PhaseTransitionEvent(basherContext, Phase.SETUP,Phase.RUN, 0 ));
    }

    public void testInvokeNullTask()
    {
        expect(_taskManager.getNextTaskExecutionContext()).andReturn( null );

        setupBasherContext(_taskRunnerImpl, 500, 0);

        expect(_randomizer.getRandomInt(500)).andReturn(1000);


        replayAll();


        Thread t = new Thread()
        {
            public void run()
            {
                try
                {
                    Thread.sleep(500);
                }
                catch (InterruptedException e)
                {
                    e.printStackTrace();
                }
                _taskRunnerImpl.stopInvoking();
            }
        };

        t.start();
        _taskRunnerImpl.run();
        verifyAll();

    }

    public void testStopRunning()
    {
        setupBasherContext(_taskRunnerImpl, 500, 0);

        _taskRunnerImpl.setTaskInvoker(new TaskInvoker()
        {
            public void invokeTask( TaskExecutionContext taskExecutionContext )
            {
                try
                {
                    Thread.sleep(200);
                }
                catch (InterruptedException e)
                {
                    e.printStackTrace();
                }
            }

            public void invokeTask(TaskExecutionContext taskExecutionContext, boolean invokeFollowers)
            {
            }
        });


        final ATestTask aTestTask = new ATestTask(100);

        expect(_taskManager.getNextTaskExecutionContext()).andReturn(new TaskExecutionContext(aTestTask));

        replayAll();

        Thread t = new Thread()
        {
            public void run()
            {
                try
                {
                    Thread.sleep(150);
                }
                catch (InterruptedException e)
                {
                    e.printStackTrace();
                }
                _taskRunnerImpl.stopInvoking();
            }
        };

        t.start();
        _taskRunnerImpl.run();
        verifyAll();

    }


    public void testInvokeWithError()
    {
        setupBasherContext(_taskRunnerImpl, 500, 0);

        final ATestTask aTestTask = new ATestTask();
        final TaskExecutionContext context = new TaskExecutionContext( aTestTask );
        expect(_taskManager.getNextTaskExecutionContext()).andReturn( context );
        expect(_taskManager.getNextTaskExecutionContext()).andReturn( null );

        final NullPointerException nullPointerException = new NullPointerException("hehe");
        _taskInvoker.invokeTask(context);
        expectLastCall().andThrow(nullPointerException);


        expect(_randomizer.getRandomInt(500)).andReturn( 1000 );

        _taskRunnerImpl.setLog(_log);
        _log.error("hehe", nullPointerException);

        replayAll();

        Thread t = new Thread()
        {
            public void run()
            {
                try
                {
                    Thread.sleep(500);
                }
                catch (InterruptedException e)
                {
                    e.printStackTrace();
                }
                _taskRunnerImpl.stopInvoking();
            }
        };

        t.start();
        _taskRunnerImpl.run();
        verifyAll();

    }

    protected void setUp() throws Exception
    {
        super.setUp();
        _basherContext = new BasherContext();
        _taskRunnerImpl = new TaskRunnerImpl();

        initializeControls();

        _taskRunnerImpl.setLog(LogFactory.getLog(TaskRunnerImpl.class));
        _taskRunnerImpl.setRandomizer(_randomizer);
        _taskRunnerImpl.setTaskManager(_taskManager);
        _taskRunnerImpl.setTaskInvoker(_taskInvoker);

    }
}
