/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.basher.internal.tasks;

import net.sourceforge.basher.BasherContext;
import net.sourceforge.basher.BasherTestCase;
import net.sourceforge.basher.ContextManager;
import net.sourceforge.basher.Scheduler;
import org.apache.commons.logging.Log;

import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;

/**
 * @author Johan Lindquist
 * @version 1.0
 */
public class TestThreadIncrementTask extends BasherTestCase
{
    private Log _log;
    private Scheduler _scheduler;
    private ContextManager _contextManager;
    private BasherContext _basherContext;
    private ThreadIncrementTask _threadIncrementTask;


    public void testInvoke()
    {
        _log.info("Incrementing thread count by '1' thread(s)");

        _log.info("Number of threads incremented by '1' thread(s)");

        expect(_scheduler.getNumberOfActiveThreads()).andReturn( 1 );
        _scheduler.addThreads(1);

        _basherContext.setThreadIncrementCount( 1 );

        expect(_contextManager.getActiveBasherContext()).andReturn( _basherContext );

        replayAll();

        try
        {
            _threadIncrementTask.run();
        }
        catch (Throwable throwable)
        {
            fail(throwable.getMessage());
        }

        verifyAll();

    }

    public void testNotInvoke()
    {
        expect(_contextManager.getActiveBasherContext()).andReturn( _basherContext );
        _log.warn("Thread increment of '0' thread(s)");

        replayAll();

        try
        {
            _threadIncrementTask.run();
        }
        catch (Throwable throwable)
        {
            fail(throwable.getMessage());
        }

        verifyAll();

    }

    public void testMaximumReached()
    {
        _log.info("Maximum threads reached, not incrementing further");

        expect(_scheduler.getNumberOfActiveThreads()).andReturn( 5 ).times( 2 );

        _basherContext.setThreadIncrementCount( 1 );
        _basherContext.setMaxNumberThreads(5);

        expect(_contextManager.getActiveBasherContext()).andReturn( _basherContext ).times( 2 );

        replayAll();

        try
        {
            _threadIncrementTask.run();
            _threadIncrementTask.run();
        }
        catch (Throwable throwable)
        {
            fail(throwable.getMessage());
        }

        verifyAll();

    }

    public void testInvokeFailed()
    {
        _log.info("Incrementing thread count by '1' thread(s)");

        _basherContext.setThreadIncrementCount( 1 );
        expect( _contextManager.getActiveBasherContext() ).andReturn( _basherContext );

        expect( _scheduler.getNumberOfActiveThreads() ).andReturn( 1 );

        final NullPointerException nullPointerException = new NullPointerException( "null" );
        _scheduler.addThreads(1);
        expectLastCall().andThrow( nullPointerException );

        _log.error(nullPointerException.getMessage(), nullPointerException);

        replayAll();

        try
        {
            _threadIncrementTask.run();
        }
        catch (Throwable throwable)
        {
            fail(throwable.getMessage());
        }

        verifyAll();

    }


    protected void setUp() throws Exception
    {
        super.setUp();
        _basherContext = new BasherContext();

        _log = createMock( Log.class );
        _scheduler = createMock(Scheduler.class);
        _contextManager = createMock( ContextManager.class );

        _threadIncrementTask = new ThreadIncrementTask();
        _threadIncrementTask.setLog(_log);
        _threadIncrementTask.setScheduler(_scheduler);
        _threadIncrementTask.setContextManager( _contextManager );

    }
}
